@@ -1,12 +1,17 @@
+from contextlib import ExitStack
 import functools
 import importlib
 import inspect
+from itertools import chain
 import os
+import shlex
+import shutil
 import sys
+import tempfile
 import traceback
 from typing import Optional, Sequence, Iterable, List, Type, Any, Callable
 from pathlib import Path
-from subprocess import check_call, run, PIPE, CompletedProcess
+from subprocess import check_call, run, PIPE, CompletedProcess, Popen
 
 import click
 
@@ -15,32 +20,26 @@ import click
 def mypy_cmd() -> Optional[Sequence[str]]:
     try:
         # preferably, use mypy from current python env
-        import mypy
-        return [sys.executable, '-m', 'mypy']
+        import mypy  # noqa: F401 fine not to use it
     except ImportError:
         pass
+    else:
+        return [sys.executable, '-m', 'mypy']
     # ok, not ideal but try from PATH
-    import shutil
     if shutil.which('mypy'):
         return ['mypy']
     warning("mypy not found, so can't check config with it. See https://github.com/python/mypy#readme if you want to install it and retry")
     return None
 
 
-from types import ModuleType
-def run_mypy(pkg: ModuleType) -> Optional[CompletedProcess]:
-    from .preinit import get_mycfg_dir
-    mycfg_dir = get_mycfg_dir()
-    # todo ugh. not sure how to extract it from pkg?
-
+def run_mypy(cfg_path: Path) -> Optional[CompletedProcess]:
     # todo dunno maybe use the same mypy config in repository?
     # I'd need to install mypy.ini then??
     env = {**os.environ}
     mpath = env.get('MYPYPATH')
-    mpath = str(mycfg_dir) + ('' if mpath is None else f':{mpath}')
+    mpath = str(cfg_path) + ('' if mpath is None else f':{mpath}')
     env['MYPYPATH'] = mpath
 
-
     cmd = mypy_cmd()
     if cmd is None:
         return None
@@ -52,7 +51,7 @@ def run_mypy(pkg: ModuleType) -> Optional[CompletedProcess]:
         '--show-error-codes',
         '--show-error-context',
         '--check-untyped-defs',
-        '-p', pkg.__name__,
+        '-p', 'my.config',
     ], stderr=PIPE, stdout=PIPE, env=env)
     return mres
 
@@ -67,6 +66,7 @@ def eprint(x: str) -> None:
 def indent(x: str) -> str:
     return ''.join('   ' + l for l in x.splitlines(keepends=True))
 
+
 OK  = '✅'
 OFF = '🔲'
 
@@ -128,10 +128,11 @@ class example:
         sys.exit(1)
 
 
-# TODO return the config as a result?
+# todo return the config as a result?
 def config_ok() -> bool:
     errors: List[Exception] = []
 
+    # at this point 'my' should already be imported, so doesn't hurt to extract paths from it
     import my
     try:
         paths: List[str] = list(my.__path__) # type: ignore[attr-defined]
@@ -142,23 +143,17 @@ def config_ok() -> bool:
     else:
         info(f'import order: {paths}')
 
-    try:
-        import my.config as cfg
-    except Exception as e:
-        errors.append(e)
-        error("failed to import the config")
-        tb(e)
-        # todo yield exception here? so it doesn't fail immediately..
-        # I guess it's fairly critical and worth exiting immediately
-        sys.exit(1)
+    # first try doing as much as possible without actually imporing my.config
+    from .preinit import get_mycfg_dir
+    cfg_path = get_mycfg_dir()
+    # alternative is importing my.config and then getting cfg_path from its __file__/__path__
+    # not sure which is better tbh
 
-    cfg_path = cfg.__file__# todo might be better to use __path__?
-    info(f"config file : {cfg_path}")
-
-    import my.core as core
+    ## check we're not using stub config
+    import my.core
     try:
-        core_pkg_path = str(Path(core.__path__[0]).parent) # type: ignore[attr-defined]
-        if cfg_path.startswith(core_pkg_path):
+        core_pkg_path = str(Path(my.core.__path__[0]).parent) # type: ignore[attr-defined]
+        if str(cfg_path).startswith(core_pkg_path):
             error(f'''
 Seems that the stub config is used ({cfg_path}). This is likely not going to work.
 See https://github.com/karlicoss/HPI/blob/master/doc/SETUP.org#setting-up-modules for more information
@@ -167,25 +162,53 @@ See https://github.com/karlicoss/HPI/blob/master/doc/SETUP.org#setting-up-module
     except Exception as e:
         errors.append(e)
         tb(e)
+    else:
+        info(f"config path : {cfg_path}")
+    ##
 
-    # todo for some reason compileall.compile_file always returns true??
-    try:
-        cmd = [sys.executable, '-m', 'compileall', str(cfg_path)]
-        check_call(cmd)
-        info('syntax check: ' + ' '.join(cmd))
-    except Exception as e:
-        errors.append(e)
+    ## check syntax
+    with tempfile.TemporaryDirectory() as td:
+        # use a temporary directory, useful because
+        # - compileall ignores -B, so always craps with .pyc files (annoyng on RO filesystems)
+        # - compileall isn't following symlinks, just silently ignores them
+        # note: ugh, annoying that copytree requires a non-existing dir before 3.8.
+        # once we have min version 3.8, can use dirs_exist_ok=True param
+        tdir = Path(td) / 'cfg'
+        # this will resolve symlinks when copying
+        shutil.copytree(cfg_path, tdir)
+        # NOTE: compileall still returns code 0 if the path doesn't exist..
+        # but in our case hopefully it's not an issue
+        cmd = [sys.executable, '-m', 'compileall', '-q', str(tdir)]
 
-    mres = run_mypy(cfg)
-    if mres is not None: # has mypy
-        rc = mres.returncode
+        try:
+            check_call(cmd)
+            info('syntax check: ' + ' '.join(cmd))
+        except Exception as e:
+            errors.append(e)
+            tb(e)
+    ##
+
+    ## check types
+    mypy_res = run_mypy(cfg_path)
+    if mypy_res is not None: # has mypy
+        rc = mypy_res.returncode
         if rc == 0:
             info('mypy check  : success')
         else:
             error('mypy check: failed')
             errors.append(RuntimeError('mypy failed'))
-            sys.stderr.write(indent(mres.stderr.decode('utf8')))
-        sys.stderr.write(indent(mres.stdout.decode('utf8')))
+            sys.stderr.write(indent(mypy_res.stderr.decode('utf8')))
+        sys.stderr.write(indent(mypy_res.stdout.decode('utf8')))
+    ##
+
+    ## finally, try actually importing the config (it should use same cfg_path)
+    try:
+        import my.config
+    except Exception as e:
+        errors.append(e)
+        error("failed to import the config")
+        tb(e)
+    ##
 
     if len(errors) > 0:
         error(f'config check: {len(errors)} errors')
@@ -239,7 +262,7 @@ def modules_check(*, verbose: bool, list_all: bool, quick: bool, for_modules: Li
             continue
 
         try:
-            mod = importlib.import_module(m)
+            mod = importlib.import_module(m)  # noqa: F841
         except Exception as e:
             # todo more specific command?
             error(f'{click.style("FAIL", fg="red")}: {m:<50} loading failed{vw}')
@@ -303,6 +326,7 @@ def tabulate_warnings() -> None:
     '''
     import warnings
     orig = warnings.formatwarning
+
     def override(*args, **kwargs) -> str:
         res = orig(*args, **kwargs)
         return ''.join('  ' + x for x in res.splitlines(keepends=True))
@@ -310,35 +334,67 @@ def tabulate_warnings() -> None:
     # TODO loggers as well?
 
 
-def _requires(module: str) -> Sequence[str]:
+def _requires(modules: Sequence[str]) -> Sequence[str]:
     from .discovery_pure import module_by_name
-    mod = module_by_name(module)
-    # todo handle when module is missing
-    r = mod.requires
-    if r is None:
-        error(f"Module {module} has no REQUIRES specification")
-        sys.exit(1)
-    return r
+    mods = [module_by_name(module) for module in modules]
+    res = []
+    for mod in mods:
+        reqs = mod.requires
+        if reqs is None:
+            error(f"Module {mod.name} has no REQUIRES specification")
+            sys.exit(1)
+        for r in reqs:
+            if r not in res:
+                res.append(r)
+    return res
 
 
-def module_requires(*, module: str) -> None:
-    rs = [f"'{x}'" for x in _requires(module)]
+def module_requires(*, module: Sequence[str]) -> None:
+    if isinstance(module, str):
+        # legacy behavior, used to take a since argument
+        module = [module]
+    rs = [f"'{x}'" for x in _requires(modules=module)]
     eprint(f'dependencies of {module}')
     for x in rs:
         click.echo(x)
 
 
-def module_install(*, user: bool, module: str) -> None:
-    # TODO hmm. not sure how it's gonna work -- presumably people use different means of installing...
-    # how do I install into the 'same' environment??
-    import shlex
-    cmd = [
-        sys.executable, '-m', 'pip', 'install',
-        *(['--user'] if user else []), # meh
-        *_requires(module),
+def module_install(*, user: bool, module: Sequence[str], parallel: bool=False) -> None:
+    if isinstance(module, str):
+        # legacy behavior, used to take a since argument
+        module = [module]
+
+    requirements = _requires(module)
+
+    pre_cmd = [
+        sys.executable, '-m', 'pip',
+        'install',
+        *(['--user'] if user else []), # todo maybe instead, forward all the remaining args to pip?
     ]
-    eprint('Running: ' + ' '.join(map(shlex.quote, cmd)))
-    check_call(cmd)
+
+    cmds = []
+    if parallel:
+        # todo not really sure if it's safe to install in parallel like this
+        # but definitely doesn't hurt to experiment for e.g. mypy pipelines
+        # pip has '--use-feature=fast-deps', but it doesn't really work
+        # I think it only helps for pypi artifacts (not git!),
+        # and only if they weren't cached
+        for r in requirements:
+            cmds.append(pre_cmd + [r])
+    else:
+        # install everything in one cmd
+        cmds.append(pre_cmd + list(requirements))
+
+    with ExitStack() as exit_stack:
+        popens = []
+        for cmd in cmds:
+            eprint('Running: ' + ' '.join(map(shlex.quote, cmd)))
+            popen = exit_stack.enter_context(Popen(cmd))
+            popens.append(popen)
+
+        for popen in popens:
+            ret = popen.wait()
+            assert ret == 0, popen
 
 
 def _ui_getchar_pick(choices: Sequence[str], prompt: str = 'Select from: ') -> int:
@@ -435,9 +491,6 @@ def query_hpi_functions(
     raise_exceptions: bool,
     drop_exceptions: bool,
 ) -> None:
-
-    from itertools import chain
-
     from .query_range import select_range, RangeTuple
 
     # chain list of functions from user, in the order they wrote them on the CLI
@@ -512,7 +565,6 @@ def main(debug: bool) -> None:
     # to avoid importing relative modules by accident during development
     # maybe can be removed later if theres more test coverage/confidence that nothing
     # would happen?
-    import tempfile
 
     # use a particular directory instead of a random one, since
     # click being decorator based means its more complicated
@@ -588,27 +640,28 @@ def module_grp() -> None:
 
 
 @module_grp.command(name='requires', short_help='print module reqs')
-@click.argument('MODULE', shell_complete=_module_autocomplete)
-def module_requires_cmd(module: str) -> None:
+@click.argument('MODULES', shell_complete=_module_autocomplete, nargs=-1, required=True)
+def module_requires_cmd(modules: Sequence[str]) -> None:
     '''
-    Print MODULE requirements
+    Print MODULES requirements
 
-    MODULE is a specific module name (e.g. my.reddit.rexport)
+    MODULES is one or more specific module names (e.g. my.reddit.rexport)
     '''
-    module_requires(module=module)
+    module_requires(module=modules)
 
 
 @module_grp.command(name='install', short_help='install module deps')
 @click.option('--user', is_flag=True, help='same as pip --user')
-@click.argument('MODULE', shell_complete=_module_autocomplete)
-def module_install_cmd(user: bool, module: str) -> None:
+@click.option('--parallel', is_flag=True, help='EXPERIMENTAL. Install dependencies in parallel.')
+@click.argument('MODULES', shell_complete=_module_autocomplete, nargs=-1, required=True)
+def module_install_cmd(user: bool, parallel: bool, modules: Sequence[str]) -> None:
     '''
-    Install dependencies for a module using pip
+    Install dependencies for modules using pip
 
-    MODULE is a specific module name (e.g. my.reddit.rexport)
+    MODULES is one or more specific module names (e.g. my.reddit.rexport)
     '''
     # todo could add functions to check specific module etc..
-    module_install(user=user, module=module)
+    module_install(user=user, module=modules, parallel=parallel)
 
 
 @main.command(name='query', short_help='query the results of a HPI function')
@@ -773,9 +826,10 @@ def query_cmd(
 
 def test_requires() -> None:
     from click.testing import CliRunner
-    result = CliRunner().invoke(main, ['module', 'requires', 'my.github.ghexport'])
+    result = CliRunner().invoke(main, ['module', 'requires', 'my.github.ghexport', 'my.browser.export'])
     assert result.exit_code == 0
     assert "github.com/karlicoss/ghexport" in result.output
+    assert "browserexport" in result.output
 
 
 if __name__ == '__main__':
