@@ -4,7 +4,7 @@ from datetime import datetime
 import functools
 from contextlib import contextmanager
 import types
-from typing import Union, Callable, Dict, Iterable, TypeVar, Sequence, List, Optional, Any, cast, Tuple, TYPE_CHECKING
+from typing import Union, Callable, Dict, Iterable, TypeVar, Sequence, List, Optional, Any, cast, Tuple, TYPE_CHECKING, NoReturn
 import warnings
 from . import warnings as core_warnings
 
@@ -12,8 +12,7 @@ from . import warnings as core_warnings
 PathIsh = Union[Path, str]
 
 # TODO only used in tests? not sure if useful at all.
-# TODO port annotations to kython?..
-def import_file(p: PathIsh, name: Optional[str]=None) -> types.ModuleType:
+def import_file(p: PathIsh, name: Optional[str] = None) -> types.ModuleType:
     p = Path(p)
     if name is None:
         name = p.stem
@@ -48,11 +47,12 @@ T = TypeVar('T')
 K = TypeVar('K')
 V = TypeVar('V')
 
+# TODO deprecate? more_itertools.one should be used
 def the(l: Iterable[T]) -> T:
     it = iter(l)
     try:
         first = next(it)
-    except StopIteration as ee:
+    except StopIteration:
         raise RuntimeError('Empty iterator?')
     assert all(e == first for e in it)
     return first
@@ -128,13 +128,6 @@ def test_make_dict() -> None:
     assert d == {0: 0, 1: 1, 2: 0, 3: 1, 4: 0}
 
 
-Cl = TypeVar('Cl')
-R = TypeVar('R')
-
-def cproperty(f: Callable[[Cl], R]) -> R:
-    return property(functools.lru_cache(maxsize=1)(f)) # type: ignore
-
-
 # https://stackoverflow.com/a/12377059/706389
 def listify(fn=None, wrapper=list):
     """
@@ -241,6 +234,7 @@ if TYPE_CHECKING:
     # I guess, later just define pass through once this is fixed: https://github.com/python/typing/issues/270
     # ok, that's actually a super nice 'pattern'
     F = TypeVar('F')
+
     class McachewType(Protocol):
         def __call__(
                 self,
@@ -280,7 +274,7 @@ def mcachew(cache_path=_cache_path_dflt, **kwargs): # type: ignore[no-redef]
         try:
             # check that it starts with 'hack' path
             Path(cache_path).relative_to(_CACHE_DIR_NONE_HACK)
-        except:
+        except: # noqa: E722 bare except
             pass # no action needed, doesn't start with 'hack' string
         else:
             # todo show warning? tbh unclear how to detect when user stopped using 'old' way and using suffix instead?
@@ -343,8 +337,7 @@ class classproperty(Generic[_R]):
 #     def __get__(self) -> _R:
 #         return self.f()
 
-# for now just serves documentation purposes... but one day might make it statically verifiable where possible?
-# TODO e.g. maybe use opaque mypy alias?
+# TODO deprecate in favor of datetime_aware
 tzdatetime = datetime
 
 
@@ -359,6 +352,8 @@ def isoparse(s: str) -> tzdatetime:
     s = s[:-1] + '+00:00'
     return datetime.fromisoformat(s)
 
+
+# legacy import -- we should use compat directly instead
 from .compat import Literal
 
 
@@ -419,6 +414,7 @@ def warn_if_empty(f: FI) -> FI: ...
 
 def warn_if_empty(f):
     from functools import wraps
+
     @wraps(f)
     def wrapped(*args, **kwargs):
         res = f(*args, **kwargs)
@@ -481,6 +477,7 @@ def _stat_iterable(it: Iterable[C], quick: bool=False) -> Any:
     total = 0
     errors = 0
     last = None
+
     def funcit():
         nonlocal errors, last, total
         for x in it:
@@ -549,7 +546,7 @@ def guess_datetime(x: Any) -> Optional[datetime]:
     # todo hmm implement withoutexception..
     try:
         d = asdict(x)
-    except:
+    except: # noqa: E722 bare except
         return None
     for k, v in d.items():
         if isinstance(v, datetime):
@@ -598,8 +595,8 @@ def asdict(thing: Any) -> Json:
     raise TypeError(f'Could not convert object {thing} to dict')
 
 
-
-
+# for now just serves documentation purposes... but one day might make it statically verifiable where possible?
+# TODO e.g. maybe use opaque mypy alias?
 datetime_naive = datetime
 datetime_aware = datetime
 
@@ -638,3 +635,12 @@ class DummyExecutor(Executor):
 
     def shutdown(self, wait: bool=True) -> None:  # type: ignore[override]
         self._shutdown = True
+
+
+# see https://hakibenita.com/python-mypy-exhaustive-checking#exhaustiveness-checking
+def assert_never(value: NoReturn) -> NoReturn:
+    assert False, f'Unhandled value: {value} ({type(value).__name__})'
+
+
+# legacy deprecated import
+from .compat import cached_property as cproperty
