@@ -1,16 +1,9 @@
 from my.core import __NOT_HPI_MODULE__
 
-from datetime import datetime
-from typing import Iterator, Optional, TYPE_CHECKING
+from typing import Iterator, Optional
 
-if TYPE_CHECKING:
-    try:
-        from typing import Protocol
-    except ImportError:
-        # requirement of mypy
-        from typing_extensions import Protocol  # type: ignore[misc]
-else:
-    Protocol = object
+from my.core.compat import Protocol
+from my.core import datetime_aware
 
 
 class Thread(Protocol):
@@ -26,7 +19,7 @@ class Message(Protocol):
     def id(self) -> str: ...
 
     @property
-    def dt(self) -> datetime: ...
+    def dt(self) -> datetime_aware: ...
 
     @property
     def text(self) -> Optional[str]: ...
@@ -46,5 +39,13 @@ def _merge_messages(*sources: Iterator[Res[Message]]) -> Iterator[Res[Message]]:
         if isinstance(r, Exception):
             return str(r)
         else:
-            return r.id
+            # use both just in case, would be easier to spot tz issues
+            # similar to twitter, might make sense to generify/document as a pattern
+            return (r.id, r.dt)
     yield from unique_everseen(chain(*sources), key=key)
+
+
+# TODO some notes about gdpr export (since there is no module yet)
+# ugh, messages seem to go from new to old in messages_N.json files as N increases :facepalm:
+# seems like it's storing local timestamp :facepalm:
+# checked against a message sent on 4 may 2022
