@@ -4,22 +4,28 @@ Twitter data (uses [[https://help.twitter.com/en/managing-your-account/how-to-do
 
 
 # before this config was named 'twitter', doesn't make too much sense for archive
-# try to import it defensively..
+# todo unify with other code like this, e.g. time.tz.via_location
 try:
     from my.config import twitter_archive as user_config
-except ImportError as e:
+except ImportError as ie:
+    if ie.name != 'twitter_archive':
+        raise ie
     try:
-        from my.config import twitter as user_config
+        from my.config import twitter as user_config # type: ignore[misc]
     except ImportError:
-        raise e # raise the original exception.. must be something else
+        raise ie # raise the original exception.. must be something else
     else:
         from ..core import warnings
         warnings.high('my.config.twitter is deprecated! Please rename it to my.config.twitter_archive in your config')
+##
 
 
 from dataclasses import dataclass
+import html
 from ..core.common import Paths, datetime_aware
+from ..core.compat import cached_property
 from ..core.error import Res
+from ..core.kompress import ZipPath
 
 @dataclass
 class twitter_archive(user_config):
@@ -38,7 +44,6 @@ from pathlib import Path
 import json
 
 from ..core.common import get_files, LazyLogger, Json
-from ..core import kompress
 
 
 
@@ -46,10 +51,10 @@ logger = LazyLogger(__name__, level="warning")
 
 
 def inputs() -> Sequence[Path]:
-    return get_files(config.export_path)[-1:]
+    return get_files(config.export_path)
 
 
-Tid = str
+from .common import TweetId, permalink
 
 
 # TODO make sure it's not used anywhere else and simplify interface
@@ -58,7 +63,7 @@ class Tweet(NamedTuple):
     screen_name: str
 
     @property
-    def id_str(self) -> str:
+    def id_str(self) -> TweetId:
         return self.raw['id_str']
 
     @property
@@ -68,11 +73,37 @@ class Tweet(NamedTuple):
 
     @property
     def permalink(self) -> str:
-        return f'https://twitter.com/{self.screen_name}/status/{self.tid}'
+        return permalink(screen_name=self.screen_name, id=self.id_str)
 
     @property
     def text(self) -> str:
-        return self.raw['full_text']
+        res: str = self.raw['full_text']
+
+        ## replace shortened URLS
+        repls = [] # from, to, what
+        for ue in self.entities['urls']:
+            [fr, to] = map(int, ue['indices'])
+            repls.append((fr, to, ue['expanded_url']))
+        # seems that media field isn't always set
+        for me in self.entities.get('media', []):
+            [fr, to] = map(int, me['indices'])
+            repls.append((fr, to, me['display_url']))
+            # todo not sure, maybe use media_url_https instead?
+            # for now doing this for compatibility with twint
+        repls = list(sorted(repls))
+        parts = []
+        idx = 0
+        for fr, to, what in repls:
+            parts.append(res[idx: fr])
+            parts.append(what)
+            idx = to
+        parts.append(res[idx:])
+        res = ''.join(parts)
+        ##
+
+        # replace stuff like &lt/&gt
+        res = html.unescape(res)
+        return res
 
     @property
     def urls(self) -> List[str]:
@@ -82,6 +113,7 @@ class Tweet(NamedTuple):
 
     @property
     def entities(self) -> Json:
+        # todo hmm what is 'extended_entities'
         return self.raw['entities']
 
     def __str__(self) -> str:
@@ -92,11 +124,11 @@ class Tweet(NamedTuple):
 
     # TODO deprecate tid?
     @property
-    def tid(self) -> Tid:
+    def tid(self) -> TweetId:
         return self.id_str
 
     @property
-    def dt(self) -> datetime:
+    def dt(self) -> datetime_aware:
         return self.created_at
 
 
@@ -104,47 +136,52 @@ class Like(NamedTuple):
     raw: Json
     screen_name: str
 
-    # TODO need to make permalink/link/url consistent across my stuff..
     @property
     def permalink(self) -> str:
         # doesn'tseem like link it export is more specific...
-        return f'https://twitter.com/{self.screen_name}/status/{self.tid}'
+        return permalink(screen_name=self.screen_name, id=self.id_str)
 
     @property
-    def id_str(self) -> Tid:
+    def id_str(self) -> TweetId:
         return self.raw['tweetId']
 
     @property
     def text(self) -> Optional[str]:
+        # NOTE: likes basically don't have anything except text and url
         # ugh. I think none means that tweet was deleted?
-        return self.raw.get('fullText')
+        res: Optional[str] = self.raw.get('fullText')
+        if res is None:
+            return None
+        res = html.unescape(res)
+        return res
 
     # TODO deprecate?
     @property
-    def tid(self) -> Tid:
+    def tid(self) -> TweetId:
         return self.id_str
 
 
-from functools import lru_cache
 class ZipExport:
     def __init__(self, archive_path: Path) -> None:
-        self.epath = archive_path
+        # todo maybe this should be insude get_files instead, perhps covered with a flag?
+        self.zpath = ZipPath(archive_path)
 
+        if (self.zpath / 'tweets.csv').exists():
+            from ..core.warnings import high
+            high("NOTE: CSV format (pre ~Aug 2018) isn't supported yet, this is likely not going to work.")
         self.old_format = False # changed somewhere around 2020.03
-        if not kompress.kexists(self.epath, 'Your archive.html'):
+        if not (self.zpath / 'Your archive.html').exists():
             self.old_format = True
 
-
-    def raw(self, what: str): # TODO Json in common?
-        logger.info('processing: %s %s', self.epath, what)
+    def raw(self, what: str) -> Iterator[Json]:
+        logger.info('processing: %s %s', self.zpath, what)
 
         path = what
         if not self.old_format:
             path = 'data/' + path
         path += '.js'
 
-        with kompress.kopen(self.epath, path) as fo:
-            ddd = fo.read()
+        ddd = (self.zpath / path).read_text()
         start = ddd.index('[')
         ddd = ddd[start:]
         for j in json.loads(ddd):
@@ -155,21 +192,23 @@ class ZipExport:
                 # older format
                 yield j
 
-    @lru_cache(1)
+    @cached_property
     def screen_name(self) -> str:
         [acc] = self.raw('account')
         return acc['username']
 
     def tweets(self) -> Iterator[Tweet]:
+        # NOTE: for some reason, created_at doesn't seem to be in order
+        # it mostly is, but there are a bunch of one-off random tweets where the time decreases (typically at the very end)
         for r in self.raw('tweet'):
-            yield Tweet(r, screen_name=self.screen_name())
+            yield Tweet(r, screen_name=self.screen_name)
 
 
     def likes(self) -> Iterator[Like]:
         # TODO ugh. would be nice to unify Tweet/Like interface
         # however, akeout only got tweetId, full text and url
         for r in self.raw('like'):
-            yield Like(r, screen_name=self.screen_name())
+            yield Like(r, screen_name=self.screen_name)
 
 
 # todo not sure about list and sorting? although can't hurt considering json is not iterative?
@@ -189,3 +228,7 @@ def stats() -> Stats:
         **stat(tweets),
         **stat(likes),
     }
+
+
+## Deprecated stuff
+Tid = TweetId
