@@ -21,12 +21,11 @@ from ..core.cfg import make_config
 config = make_config(twint)
 
 
-from datetime import datetime
+from datetime import datetime, timezone
 from typing import NamedTuple, Iterator, List
 from pathlib import Path
 
 from ..core.common import get_files, LazyLogger, Json, datetime_aware
-from ..core.time import abbr_to_timezone
 
 log = LazyLogger(__name__)
 
@@ -35,29 +34,49 @@ def get_db_path() -> Path:
     return max(get_files(config.export_path))
 
 
+from .common import TweetId, permalink
+
+
 class Tweet(NamedTuple):
     row: Json
 
     @property
-    def id_str(self) -> str:
+    def id_str(self) -> TweetId:
         return self.row['id_str']
 
     @property
     def created_at(self) -> datetime_aware:
         seconds = self.row['created_at'] / 1000
-        tz_abbr = self.row['timezone']
-        tz = abbr_to_timezone(tz_abbr)
-        dt = datetime.fromtimestamp(seconds, tz=tz)
-        return dt
+        tz = timezone.utc
+        # NOTE: UTC seems to be the case at least for the older version of schema I was using
+        # in twint, it was extracted from "data-time-ms" field in the scraped HML
+        # https://github.com/twintproject/twint/blob/e3345426eb24154ff084be22e4fed5cfa4631930/twint/tweet.py#L85
+        #
+        # I checked against twitter archive which is definitely UTC, and it seems to match
+        # also seems that other people are treating it as utc, e.g.
+        # https://github.com/thomasancheriyil/Red-Tide-Detection-based-on-Twitter/blob/beb200be60cc66dcbc394e670513715509837812/python/twitterGapParse.py#L61-L62
+        #
+        # twint is also saving 'timezone', but this is local machine timezone at the time of scraping?
+        # perhaps they thought date-time-ms was local time... or just kept it just in case (they are keepin lots on unnecessary stuff in the db)
+        return datetime.fromtimestamp(seconds, tz=tz)
 
-    # TODO permalink -- take user into account?
     @property
     def screen_name(self) -> str:
         return self.row['screen_name']
 
     @property
     def text(self) -> str:
-        return self.row['tweet']
+        text = self.row['tweet']
+        mentions_s = self.row['mentions']
+        if len(mentions_s) > 0:
+            # at some point for no apparent reasions mentions stopped appearing from tweet text in twint
+            # note that the order is still inconsisnent against twitter archive, but not much we can do
+            mentions = mentions_s.split(',')
+            for m in mentions:
+                # ugh. sometimes they appear as lowercase in text, sometimes not..
+                if m.lower() not in text.lower():
+                    text = f'@{m} ' + text
+        return text
 
     @property
     def urls(self) -> List[str]:
@@ -66,10 +85,9 @@ class Tweet(NamedTuple):
             return []
         return ustr.split(',')
 
-    # TODO move to common
     @property
     def permalink(self) -> str:
-        return f'https://twitter.com/{self.screen_name}/status/{self.id_str}'
+        return permalink(screen_name=self.screen_name, id=self.id_str)
 
 
     # TODO urls
