@@ -1,16 +1,15 @@
-#!/usr/bin/env python3
+from .common import skip_if_not_karlicoss as pytestmark
+
 from more_itertools import ilen
-
-from my.coding.github import get_events
-
 # todo test against stats? not sure.. maybe both
 
-def test_gdpr():
+def test_gdpr() -> None:
     import my.github.gdpr as gdpr
     assert ilen(gdpr.events()) > 100
 
 
-def test():
+def test() -> None:
+    from my.coding.github import get_events
     events = get_events()
     assert ilen(events) > 100
     for e in events:
